package app.customizer;

import java.lang.reflect.Method;
import java.util.Map;

import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;

import org.seasar.framework.container.ComponentDef;
import org.seasar.framework.container.customizer.AbstractCustomizer;
import org.seasar.framework.container.factory.AspectDefFactory;
import org.seasar.framework.util.StringUtil;
import org.seasar.framework.util.tiger.CollectionsUtil;

public class Ejb3TxAttributeCustomizer extends AbstractCustomizer {

    protected static final Map<TransactionAttributeType, String> txInterceptors = CollectionsUtil
            .newHashMap();
    static {
        txInterceptors.put(TransactionAttributeType.MANDATORY,
                "ejb3tx.mandatoryTx");
        txInterceptors
                .put(TransactionAttributeType.REQUIRED, "ejb3tx.requiredTx");
        txInterceptors.put(TransactionAttributeType.REQUIRES_NEW,
                "ejb3tx.requiresNewTx");
        txInterceptors.put(TransactionAttributeType.NOT_SUPPORTED,
                "ejb3tx.notSupportedTx");
        txInterceptors.put(TransactionAttributeType.NEVER, "ejb3tx.neverTx");
    }

    protected TransactionAttributeType defaultAttributeType = TransactionAttributeType.REQUIRED;

    public Ejb3TxAttributeCustomizer() {
    }


    public void setDefaultAttributeType(
            final TransactionAttributeType defaultAttributeType) {
        this.defaultAttributeType = defaultAttributeType;
    }

    @Override
    protected void doCustomize(final ComponentDef componentDef) {
        final Class<?> componentClass = componentDef.getComponentClass();
        final TransactionAttribute classAttribute = componentClass
                .getAnnotation(TransactionAttribute.class);
        final TransactionAttributeType classAttributeType = classAttribute != null ? classAttribute
                .value()
                : defaultAttributeType;
        for (final Method method : componentClass.getMethods()) {
            if (method.isSynthetic() || method.isBridge()) {
                continue;
            }
            if (method.getDeclaringClass() == Object.class) {
                continue;
            }
            final TransactionAttribute methodAttribute = method
                    .getAnnotation(TransactionAttribute.class);
            final TransactionAttributeType methodAttributeType = methodAttribute != null ? methodAttribute
                    .value()
                    : classAttributeType;
            final String interceptorName = txInterceptors
                    .get(methodAttributeType);
            if (!StringUtil.isEmpty(interceptorName)) {
                componentDef.addAspectDef(AspectDefFactory.createAspectDef(
                        interceptorName, method));
            }
        }
    }

}
